@@ -100,19 +100,19 @@ class Event(object):
                  username=prop.username, job_id=0, title='',
                  timedate=0):
 
-        self.device_uri = unicode(utils.xrstrip(device_uri, '\x00'))[:64].encode('utf-8')
-        self.printer_name = unicode(utils.xrstrip(printer_name, '\x00'))[:64].encode('utf-8')
+        self.device_uri = unicode(utils.xrstrip(device_uri, '\x00'))[:128].encode('utf-8')
+        self.printer_name = unicode(utils.xrstrip(printer_name, '\x00'))[:128].encode('utf-8')
         self.event_code = int(event_code)
         self.username = unicode(utils.xrstrip(username, '\x00'))[:32].encode('utf-8')
         self.job_id = int(job_id)
-        self.title = unicode(utils.xrstrip(title, '\x00'))[:64].encode('utf-8')
+        self.title = unicode(utils.xrstrip(title, '\x00'))[:128].encode('utf-8')
 
         if timedate:
             self.timedate = float(timedate)
         else:
             self.timedate = time.time()
 
-        self.pipe_fmt = "64s64sI32sI64sf"
+        self.pipe_fmt = "80s80sI32sI80sf"
         self.dbus_fmt = "ssisisd"
 
 
@@ -168,7 +168,7 @@ class FaxEvent(Event):
     def __init__(self, temp_file, event):
         Event.__init__(self, *event.as_tuple())
         self.temp_file = temp_file
-        self.pipe_fmt = "64s64sI32sI64sfs"
+        self.pipe_fmt = "80s80sI32sI80sfs"
         self.dbus_fmt = "ssisisfs"
 
 
@@ -193,7 +193,7 @@ class DeviceIOEvent(Event):
     def __init__(self, bytes_written, event):
         Event.__init__(self, *event.as_tuple())
         self.bytes_written = bytes_written
-        self.pipe_fmt = "64s64sI32sI64sfI"
+        self.pipe_fmt = "80s80sI32sI80sfI"
         self.dbus_fmt = "ssisisfi"
 
 
@@ -945,6 +945,7 @@ def queryString(string_id, typ=0):
 
 AGENT_types = { AGENT_TYPE_NONE        : 'invalid',
                 AGENT_TYPE_BLACK       : 'black',
+                AGENT_TYPE_BLACK_B8800 : 'black',
                 AGENT_TYPE_CMY         : 'cmy',
                 AGENT_TYPE_KCM         : 'kcm',
                 AGENT_TYPE_CYAN        : 'cyan',
@@ -1120,7 +1121,7 @@ class Device(object):
             'dev-file'         : self.dev_file,
             'host'             : self.host,
             'port'             : self.port,
-            'cups-printers'    : ','.join(self.cups_printers),
+            'cups-printers'    : self.cups_printers,
             'status-code'      : self.status_code,
             'status-desc'      : '',
             'deviceid'         : '',
@@ -1328,9 +1329,15 @@ class Device(object):
     def openSoapFax(self):
         return self.__openChannel(hpmudext.HPMUD_S_SOAP_FAX)
 
+    def openMarvellFax(self):
+        return self.__openChannel(hpmudext.HPMUD_S_MARVELL_FAX_CHANNEL)
+
     def closeSoapFax(self):
         return self.__closeChannel(hpmudext.HPMUD_S_SOAP_FAX)
 
+    def closeMarvellFax(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_MARVELL_FAX_CHANNEL)
+
     def closeWifiConfig(self):
         return self.__closeChannel(hpmudext.HPMUD_S_WIFI_CHANNEL)
 
@@ -1522,7 +1529,7 @@ class Device(object):
 
                 self.dq.update({
                     'serial'           : self.serial,
-                    'cups-printers'    : ','.join(self.cups_printers),
+                    'cups-printers'    : self.cups_printers,
                     'status-code'      : self.status_code,
                     'status-desc'      : status_desc,
                     'deviceid'         : self.raw_deviceID,
@@ -1600,7 +1607,7 @@ class Device(object):
                 if self.io_state == IO_STATE_NON_HP:
                     self.model = p.makemodel.split(',')[0]
 
-        self.dq.update({'cups-printers' : ','.join(self.cups_printers)})
+        self.dq.update({'cups-printers' : self.cups_printers})
 
         try:
             self.first_cups_printer = self.cups_printers[0]
@@ -1652,7 +1659,7 @@ class Device(object):
 
             self.dq.update({
                 'serial'           : self.serial,
-                'cups-printers'    : ','.join(self.cups_printers),
+                'cups-printers'    : self.cups_printers,
                 'status-code'      : self.status_code,
                 'status-desc'      : status_desc,
                 'deviceid'         : self.raw_deviceID,
@@ -1685,6 +1692,10 @@ class Device(object):
                 log.debug("Type 8: LJ PJL")
                 status_block = status.StatusType8(self)
 
+            elif status_type == STATUS_TYPE_LEDM:
+                log.debug("Type 10: LEDM")
+                status_block = status.StatusType10(self)
+
             else:
                 log.error("Unimplemented status type: %d" % status_type)
 
@@ -2091,6 +2102,9 @@ class Device(object):
     def readSoapFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openSoapFax, bytes_to_read, stream, timeout, allow_short_read)
 
+    def readMarvellFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openMarvellFax, bytes_to_read, stream, timeout, allow_short_read)
+
     def readWifiConfig(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openWifiConfig, bytes_to_read, stream, timeout, allow_short_read)
 
@@ -2171,6 +2185,9 @@ class Device(object):
     def writeSoapFax(self, data):
         return self.__writeChannel(self.openSoapFax, data)
 
+    def writeMarvellFax(self, data):
+        return self.__writeChannel(self.openMarvellFax, data)
+
     def writeWifiConfig(self, data):
         return self.__writeChannel(self.openWifiConfig, data)
 
