@@ -8,7 +8,7 @@
 # All Rights Reserved
 #
 # Modifications by Don Welch
-# (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
 #
 # Redistribution and use in source and binary forms, with or without modification,
 # are permitted provided that the following conditions are met:
@@ -255,10 +255,9 @@ magic = [
     [0L, 'string', '=', '#! /usr/bin/python', 'application/x-python'],
     [0L, 'string', '=', '#! /usr/bin/python', 'application/x-python'],
     [0L, 'string', '=', 'eval "exec /usr/bin/python', 'application/x-python'],
-    [0L, 'string', '=', '#!/usr/local/bin/python', 'application/x-python'],
-    [0L, 'string', '=', '#! /usr/local/bin/python', 'application/x-python'],
-    [0L, 'string', '=', '#! /usr/local/bin/python', 'application/x-python'],
-    [0L, 'string', '=', 'eval "exec /usr/local/bin/python', 'application/x-python'],
+    [0L, 'string', '=', '#!/usr/local/bin/python2.5', 'application/x-python'],
+    [0L, 'string', '=', '#! /usr/local/bin/python2.5', 'application/x-python'],
+    [0L, 'string', '=', 'eval "exec /usr/local/bin/python2.5', 'application/x-python'],
     [0L, 'string', '=', '#!/usr/local/bin/python2.5', 'application/x-python'],
     [0L, 'string', '=', '#! /usr/local/bin/python2.5', 'application/x-python'],
     [0L, 'string', '=', '#!/bin/rc', 'text/script'],
@@ -889,7 +888,7 @@ magic = [
     [0L, 'long', '=', 270L, 'unknown readable demand paged pure executable'],
     [0L, 'string', '=', 'begin uuencoded', 'or xxencoded text'],
     [0L, 'string', '=', 'xbtoa Begin', "btoa'd text"],
-    [0L, 'string', '=', '$\012ship', "ship'd binary text"],
+    [0L, 'string', '=', '$012ship', "ship'd binary text"],
     [0L, 'string', '=', 'Decode the following with bdeco', 'bencoded News text'],
     [11L, 'string', '=', 'must be converted with BinHex', 'BinHex binary text'],
     [0L, 'short', '=', 610L, 'Perkin-Elmer executable'],
