@@ -883,7 +883,7 @@ else: # INTERACTIVE_MODE
                 if len(path) > 0:
                     cmd = 'hp-testpage %s' % param
                 else:
-                    cmd = '${MODPY_BIN} ./testpage.py %s' % param
+                    cmd = '/usr/local/bin/python2.5 ./testpage.py %s' % param
 
                 log.debug(cmd)
 
